@@ -13,12 +13,13 @@ try:
 except ImportError:
     requests_utils = None
 
-__version__ = "7.0.38"
+__version__ = "7.0.40"
 
-os.environ.setdefault(
-    'TRYTOND_APPNAME',
-    os.path.basename(getattr(__main__, '__file__', 'trytond')))
-os.environ.setdefault('TRYTOND_TZ', os.environ.get('TZ', 'UTC'))
+if not os.environ.get('TRYTOND_APPNAME'):
+    os.environ['TRYTOND_APPNAME'] = os.path.basename(
+        getattr(__main__, '__file__', 'trytond'))
+if not os.environ.get('TRYTOND_TZ'):
+    os.environ['TRYTOND_TZ'] = os.environ.get('TZ') or 'UTC'
 os.environ['TZ'] = 'UTC'
 if hasattr(time, 'tzset'):
     time.tzset()
