@@ -588,7 +588,12 @@ class Database(DatabaseInterface):
         # This call is not thread safe
         return cursor.lastrowid
 
-    def lock(self, connection, table):
+    @classmethod
+    def lock(cls, connection, table):
+        pass
+
+    @classmethod
+    def lock_records(cls, connection, table, ids):
         pass
 
     def lock_id(self, id, timeout=None):
