@@ -4,6 +4,7 @@ import configparser
 import logging
 import os
 import urllib.parse
+from getpass import getuser
 
 __all__ = ['config', 'get_hostname', 'get_port', 'split_netloc',
     'parse_listen', 'parse_uri']
@@ -60,19 +61,25 @@ class TrytonConfigParser(configparser.ConfigParser):
         self.set('database', 'retry', '5')
         self.set('database', 'language', 'en')
         self.set('database', 'timeout', str(30 * 60))
+        self.set('database', 'subquery_threshold', str(1_000))
         self.add_section('request')
         self.set('request', 'max_size', str(2 * 1024 * 1024))
         self.set('request', 'max_size_authenticated',
             str(2 * 1024 * 1024 * 1024))
         self.add_section('cache')
+        self.set('cache', 'transaction', '10')
         self.set('cache', 'model', '200')
         self.set('cache', 'record', '2000')
         self.set('cache', 'field', '100')
+        self.set('cache', 'default', '1024')
+        self.set('cache', 'ir.message', '10240')
+        self.set('cache', 'ir.translation', '10240')
         self.add_section('queue')
         self.set('queue', 'worker', 'False')
         self.add_section('ssl')
         self.add_section('email')
         self.set('email', 'uri', 'smtp://localhost:25')
+        self.set('email', 'from', getuser())
         self.add_section('session')
         self.set('session', 'authentications', 'password')
         self.set('session', 'max_age', str(60 * 60 * 24 * 30))
@@ -83,7 +90,6 @@ class TrytonConfigParser(configparser.ConfigParser):
         self.set('session', 'ip_network_6', '56')
         self.add_section('password')
         self.set('password', 'length', '8')
-        self.set('password', 'entropy', '0.75')
         self.set('password', 'reset_timeout', str(24 * 60 * 60))
         self.add_section('bus')
         self.set('bus', 'allow_subscribe', 'False')
@@ -102,6 +108,8 @@ class TrytonConfigParser(configparser.ConfigParser):
                 section, option = key[len('TRYTOND_'):].lower().split('__', 1)
             except ValueError:
                 continue
+            if section.startswith('wsgi_'):
+                section = section.replace('wsgi_', 'wsgi ')
             if not self.has_section(section):
                 self.add_section(section)
             self.set(section, option, value)
