@@ -1090,8 +1090,13 @@ class ActionWizard(ActionMixin, ModelSQL, ModelView):
         # TODO add cache
         actions = cls.search([
                 (cls._action_name, '=', name),
-                ('model', '=', model),
-                ], limit=1)
+                ['OR',
+                    ('model', '=', model),
+                    ('model', '=', None),
+                    ],
+                ],
+            order=[('model', 'ASC NULLS LAST')],
+            limit=1)
         if actions:
             action, = actions
             return action.name
