@@ -1,17 +1,19 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
-from .modelstorage import (AccessError, ImportDataError, ValidationError,
-    DomainValidationError, RequiredValidationError, SizeValidationError,
-    DigitsValidationError, ForbiddenCharValidationError,
-    SelectionValidationError, TimeFormatValidationError)
 from .modelsql import ForeignKeyError, SQLConstraintError
-from .modelview import AccessButtonError
+from .modelstorage import (
+    AccessError, DigitsValidationError, DomainValidationError,
+    ForbiddenCharValidationError, ImportDataError, RequiredValidationError,
+    SelectionValidationError, SizeValidationError, TimeFormatValidationError,
+    ValidationError)
+from .modelview import AccessButtonError, ButtonActionException
 from .tree import RecursionError
 
 __all__ = [
     AccessButtonError,
     AccessError,
+    ButtonActionException,
     DigitsValidationError,
     DomainValidationError,
     ForeignKeyError,
