@@ -1,22 +1,23 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import warnings
-from functools import partial, wraps
+from functools import wraps
 
 import sql
-from sql import (operators, Column, Literal, Select, CombiningQuery, Null,
-    Query, Expression, Cast)
+from sql import (
+    Cast, Column, CombiningQuery, Expression, Literal, Null, Query, Select,
+    operators)
 from sql.aggregate import Min
 from sql.conditionals import Coalesce, NullIf
 from sql.operators import Concat
 
 from trytond import backend
-from trytond.cache import LRUDictTransaction
 from trytond.const import OPERATORS
 from trytond.pool import Pool
-from trytond.pyson import PYSON, PYSONEncoder, PYSONDecoder, Eval
+from trytond.pyson import PYSON, Eval, PYSONDecoder, PYSONEncoder
 from trytond.rpc import RPC
-from trytond.tools.string_ import StringPartitioned, LazyString
+from trytond.tools import cached_property
+from trytond.tools.string_ import LazyString, StringPartitioned
 from trytond.transaction import Transaction
 
 _sql_version = tuple(map(int, sql.__version__.split('.')))
@@ -38,7 +39,7 @@ def domain_validate(value):
                                 and arg[1] in OPERATORS)
                         or (
                                 isinstance(arg[1], PYSON)
-                                and arg[1].types() == set([str]))))):
+                                and arg[1].types() == {str})))):
                 pass
             elif isinstance(arg, list):
                 if not test_domain(arg):
@@ -57,12 +58,12 @@ def states_validate(value):
         assert isinstance(value[state], (bool, PYSON)), \
             'values of states must be PYSON'
         if hasattr(value[state], 'types'):
-            assert value[state].types() == set([bool]), \
+            assert value[state].types() == {bool}, \
                 'values of states must return boolean'
 
 
 def depends_validate(value):
-    assert isinstance(value, list), 'depends must be a list'
+    assert isinstance(value, set), 'depends must be a set'
 
 
 def context_validate(value):
@@ -73,7 +74,7 @@ def size_validate(value):
     if value is not None:
         assert isinstance(value, (int, PYSON)), 'size must be PYSON'
         if hasattr(value, 'types'):
-            assert value.types() == set([int]), \
+            assert value.types() <= {int, type(None)}, \
                 'size must return integer'
 
 
@@ -81,7 +82,7 @@ def search_order_validate(value):
     if value is not None:
         assert isinstance(value, (list, PYSON)), 'search_order must be PYSON'
         if hasattr(value, 'types'):
-            assert value.types() == set([list]), 'search_order must be PYSON'
+            assert value.types() == {list}, 'search_order must be PYSON'
 
 
 def _set_value(record, field):
@@ -108,12 +109,12 @@ def depends(*fields, **kwargs):
 
     def decorator(func):
         depends = getattr(func, 'depends', set())
-        depends |= set(fields)
+        depends.update(fields)
         setattr(func, 'depends', depends)
 
         if methods:
             depend_methods = getattr(func, 'depend_methods', set())
-            depend_methods |= set(methods)
+            depend_methods.update(methods)
             setattr(func, 'depend_methods', depend_methods)
 
         @wraps(func)
@@ -125,32 +126,33 @@ def depends(*fields, **kwargs):
     return decorator
 
 
+def _iter_eval_fields(value):
+    "Iterate over evaluated fields"
+    if isinstance(value, Eval):
+        yield value.basename
+    elif isinstance(value, PYSON):
+        yield from _iter_eval_fields(value.pyson())
+    elif isinstance(value, (list, tuple)):
+        for val in value:
+            yield from _iter_eval_fields(val)
+    elif isinstance(value, dict):
+        for val in value.values():
+            yield from _iter_eval_fields(val)
+
+
 def get_eval_fields(value):
     "Return fields evaluated"
-    class Encoder(PYSONEncoder):
-        def __init__(self, *args, **kwargs):
-            super(Encoder, self).__init__(*args, **kwargs)
-            self.fields = set()
-
-        def default(self, obj):
-            if isinstance(obj, Eval):
-                fname = obj._value
-                if not fname.startswith('_parent_'):
-                    self.fields.add(fname)
-            return super(Encoder, self).default(obj)
-
-    encoder = Encoder()
-    encoder.encode(value)
-    return encoder.fields
+    fields = set(_iter_eval_fields(value))
+    fields.discard('context')  # TODO: remove when context is renamed
+    return fields
 
 
 def instanciate_values(Target, value, **extra):
-    from ..modelstorage import ModelStorage, cache_size
+    from ..modelstorage import ModelStorage, local_cache
     kwargs = {}
     ids = []
     if issubclass(Target, ModelStorage):
-        kwargs['_local_cache'] = LRUDictTransaction(
-            cache_size(), Target._record)
+        kwargs['_local_cache'] = local_cache(Target)
         kwargs['_ids'] = ids
 
     def instance(data):
@@ -193,21 +195,49 @@ def on_change_result(record):
     return record._changed_values
 
 
-def with_inactive_records(func):
+def on_change_with_result(fieldname):
+    def decorator(func):
+        @wraps(func)
+        def wrapper(self, *args, **kwargs):
+            value = func(self, *args, **kwargs)
+            setattr(self, fieldname, value)
+            return self._changed_values
+        return wrapper
+    return decorator
+
+
+def domain_method(func):
     @wraps(func)
-    def wrapper(*args, **kwargs):
-        with Transaction().set_context(active_test=False):
-            return func(*args, **kwargs)
+    def wrapper(self, domain, tables, Model):
+        name = domain[0].split('.', 1)[0]
+        assert name == self.name
+        method = getattr(Model, f'domain_{name}', None)
+        if method:
+            return method(domain, tables)
+        return func(self, domain, tables, Model)
+    return wrapper
+
+
+def order_method(func):
+    @wraps(func)
+    def wrapper(self, name, tables, Model):
+        fname, _, oexpr = name.partition('.')
+        assert fname == self.name
+        method = getattr(Model, f'order_{fname}', None)
+        if not oexpr and method:
+            return method(tables)
+        else:
+            return func(self, name, tables, Model)
     return wrapper
 
 
 SQL_OPERATORS = {
     '=': operators.Equal,
     '!=': operators.NotEqual,
-    'like': partial(operators.Like, escape='\\'),
-    'not like': partial(operators.NotLike, escape='\\'),
-    'ilike': partial(operators.ILike, escape='\\'),
-    'not ilike': partial(operators.NotILike, escape='\\'),
+    'like': operators.Like,
+    'not like': operators.NotLike,
+    'ilike': operators.ILike,
+    'not ilike': operators.NotILike,
     'in': operators.In,
     'not in': operators.NotIn,
     '<=': operators.LessEqual,
@@ -223,7 +253,7 @@ class Field(object):
     _py_type = None
 
     def __init__(self, string='', help='', required=False, readonly=False,
-            domain=None, states=None, select=False, on_change=None,
+            domain=None, states=None, on_change=None,
             on_change_with=None, depends=None, context=None,
             loading='eager'):
         '''
@@ -237,7 +267,6 @@ class Field(object):
             ``readonly`` and ``invisible``. Values are pyson expressions that
             will be evaluated with record values. This allows to change
             dynamically the attributes of the field.
-        :param select: An boolean. When True search will be optimized.
         :param on_change: A list of values. If set, the client will call the
             method ``on_change_<field_name>`` when the user changes the field
             value. It then passes this list of values as arguments to the
@@ -245,7 +274,7 @@ class Field(object):
         :param on_change_with: A list of values. Like ``on_change``, but
             defined the other way around. The list contains all the fields that
             must update the current field.
-        :param depends: A list of field name on which this one depends.
+        :param depends: A set of field name on which this one depends.
         :param context: A dictionary which will be given to open the relation
             fields.
         :param loading: Define how the field must be loaded:
@@ -258,24 +287,23 @@ class Field(object):
         self.required = required
         self.readonly = readonly
         self.__domain = None
-        self.domain = domain or []
+        self.domain = domain
         self.__states = None
         self.states = states or {}
-        self.select = bool(select)
         self.on_change = set()
         if on_change:
             warnings.warn('on_change argument is deprecated, '
                 'use the depends decorator',
                 DeprecationWarning, stacklevel=3)
-            self.on_change |= set(on_change)
+            self.on_change.update(on_change)
         self.on_change_with = set()
         if on_change_with:
             warnings.warn('on_change_with argument is deprecated, '
                 'use the depends decorator',
                 DeprecationWarning, stacklevel=3)
-            self.on_change_with |= set(on_change_with)
+            self.on_change_with.update(on_change_with)
         self.__depends = None
-        self.depends = depends or []
+        self.depends = depends or set()
         self.__context = None
         self.context = context or {}
         assert loading in ('lazy', 'eager'), \
@@ -303,6 +331,8 @@ class Field(object):
         return self.__domain
 
     def _set_domain(self, value):
+        if value is None:
+            value = []
         domain_validate(value)
         self.__domain = value
 
@@ -321,11 +351,30 @@ class Field(object):
         return self.__depends
 
     def _set_depends(self, value):
+        value = set(value)
         depends_validate(value)
         self.__depends = value
 
     depends = property(_get_depends, _set_depends)
 
+    @cached_property
+    def display_depends(self):
+        depends = get_eval_fields(self.states.get('invisible'))
+        return self.depends | depends
+
+    @cached_property
+    def edition_depends(self):
+        depends = get_eval_fields(self.domain)
+        depends |= get_eval_fields(self.states.get('readonly'))
+        depends |= get_eval_fields(self.states.get('required'))
+        return self.depends | depends
+
+    @cached_property
+    def validation_depends(self):
+        depends = get_eval_fields(self.domain)
+        depends |= get_eval_fields(self.states.get('required'))
+        return self.depends | depends
+
     def _get_context(self):
         return self.__context
 
@@ -345,8 +394,13 @@ class Field(object):
 
     def __set__(self, inst, value):
         assert self.name is not None
+        if isinstance(value, (Query, Expression)):
+            raise ValueError("Can not assign SQL")
         if inst._values is None:
             inst._values = inst._record()
+        if (self._py_type and value is not None
+                and not isinstance(value, self._py_type)):
+            value = self._py_type(value)
         inst._values[self.name] = value
 
     def sql_format(self, value):
@@ -391,14 +445,11 @@ class Field(object):
                     expression &= (column != Null)
         return expression
 
+    @domain_method
     def convert_domain(self, domain, tables, Model):
         "Return a SQL expression for the domain using tables"
         table, _ = tables[None]
         name, operator, value = domain
-        assert name == self.name
-        method = getattr(Model, 'domain_%s' % name, None)
-        if method:
-            return method(domain, tables)
         Operator = SQL_OPERATORS[operator]
         column = self.sql_column(table)
         column = self._domain_column(operator, column)
@@ -410,20 +461,16 @@ class Field(object):
         expression = self._domain_add_null(column, operator, value, expression)
         return expression
 
+    @order_method
     def convert_order(self, name, tables, Model):
         "Return a SQL expression to order"
-        assert name == self.name
         table, _ = tables[None]
-        method = getattr(Model, 'order_%s' % name, None)
-        if method:
-            return method(tables)
-        else:
-            return [self.sql_column(table)]
+        return [self.sql_column(table)]
 
     def set_rpc(self, model):
-        for attribute, result in (
-                ('on_change', on_change_result),
-                ('on_change_with', None),
+        for attribute, decorator, result in (
+                ('on_change', None, on_change_result),
+                ('on_change_with', on_change_with_result(self.name), None),
                 ):
             if not getattr(self, attribute):
                 continue
@@ -431,7 +478,8 @@ class Field(object):
             assert hasattr(model, func_name), \
                 'Missing %s on model %s' % (func_name, model.__name__)
             model.__rpc__.setdefault(
-                func_name, RPC(instantiate=0, result=result))
+                func_name,
+                RPC(instantiate=0, decorator=decorator, result=result))
 
     def definition(self, model, language):
         pool = Pool()
@@ -441,7 +489,6 @@ class Field(object):
             'context': encoder.encode(self.context),
             'loading': self.loading,
             'name': self.name,
-            'depends': self.depends,
             'on_change': list(self.on_change),
             'on_change_with': list(self.on_change_with),
             'readonly': self.readonly,
@@ -449,8 +496,8 @@ class Field(object):
             'states': encoder.encode(self.states),
             'type': self._type,
             'domain': encoder.encode(self.domain),
-            'searchable': hasattr(model, 'search'),
-            'sortable': hasattr(model, 'search'),
+            'searchable': self.searchable(model),
+            'sortable': self.sortable(model),
             }
 
         # Add id to on_change's if they are not cached
@@ -503,6 +550,12 @@ class Field(object):
                     translations.append((name, ttype, language, source))
         return translations
 
+    def searchable(self, model):
+        return hasattr(model, 'search')
+
+    def sortable(self, model):
+        return hasattr(model, 'search')
+
 
 class FieldTranslate(Field):
 
@@ -613,6 +666,7 @@ class FieldTranslate(Field):
             language = get_parent_language(language)
         return table, join, column
 
+    @domain_method
     def convert_domain(self, domain, tables, Model):
         if not self.translate:
             return super(FieldTranslate, self).convert_domain(
@@ -668,6 +722,7 @@ class FieldTranslate(Field):
             language = get_parent_language(language)
         return column
 
+    @order_method
     def convert_order(self, name, tables, Model):
         if not self.translate:
             return super().convert_order(name, tables, Model)
