@@ -3,16 +3,19 @@
 from collections import defaultdict
 from itertools import chain
 
-from sql import Cast, Literal, Null
-from sql.functions import Substring, Position
+from sql import Literal, Null
 from sql.conditionals import Coalesce
 
 from trytond.pool import Pool
 from trytond.pyson import PYSONEncoder
-from trytond.tools import grouped_slice
+from trytond.tools import cached_property, grouped_slice
 from trytond.transaction import Transaction
-from .field import (Field, size_validate, instanciate_values, domain_validate,
-    search_order_validate, context_validate, instantiate_context)
+
+from .field import (
+    Field, context_validate, domain_method, domain_validate, get_eval_fields,
+    instanciate_values, instantiate_context, search_order_validate,
+    size_validate)
+from .function import Function
 
 
 class Many2Many(Field):
@@ -40,11 +43,6 @@ class Many2Many(Field):
         :param search_context: The context to use when searching for a record
         :param filter: A domain to filter target records.
         '''
-        if datetime_field:
-            if depends:
-                depends.append(datetime_field)
-            else:
-                depends = [datetime_field]
         super(Many2Many, self).__init__(string=string, help=help,
             required=required, readonly=readonly, domain=domain, states=states,
             on_change=on_change, on_change_with=on_change_with,
@@ -106,6 +104,25 @@ class Many2Many(Field):
     def add_remove(self):
         return self.domain
 
+    @cached_property
+    def display_depends(self):
+        depends = super().display_depends
+        if self.datetime_field:
+            depends.add(self.datetime_field)
+        return depends
+
+    @cached_property
+    def edition_depends(self):
+        depends = super().edition_depends
+        depends |= get_eval_fields(self.size)
+        return depends
+
+    @cached_property
+    def validation_depends(self):
+        depends = super().validation_depends
+        depends |= get_eval_fields(self.size)
+        return depends
+
     def sql_type(self):
         return None
 
@@ -121,14 +138,19 @@ class Many2Many(Field):
         for i in ids:
             res[i] = []
 
-        if self.order is None:
-            order = [(self.target, 'ASC')]
-        else:
-            order = self.order
-
         Relation = self.get_relation()
         origin_field = Relation._fields[self.origin]
 
+        if (not isinstance(origin_field, Function)
+                or hasattr(Relation, 'order_' + self.field)):
+            order = [(self.origin, None)]
+        else:
+            order = []
+        if self.order is None:
+            order += [(self.target, None)]
+        else:
+            order += self.order
+
         relations = []
         for sub_ids in grouped_slice(ids):
             if origin_field._type == 'reference':
@@ -140,7 +162,7 @@ class Many2Many(Field):
             if self.filter:
                 clause.append((self.target, 'where', self.filter))
             relations.append(Relation.search(clause, order=order))
-        relations = list(chain(*relations))
+        relations = Relation.browse(list(chain(*relations)))
 
         for relation in relations:
             origin_id = getattr(relation, self.origin).id
@@ -332,6 +354,7 @@ class Many2Many(Field):
             return ~expression
         return expression
 
+    @domain_method
     def convert_domain(self, domain, tables, Model):
         from ..modelsql import convert_from
         pool = Pool()
@@ -356,9 +379,7 @@ class Many2Many(Field):
         origin_where = None
         if origin_field._type == 'reference':
             origin_where = origin.like(Model.__name__ + ',%')
-            origin = Cast(Substring(origin,
-                    Position(',', origin) + Literal(1)),
-                Relation.id.sql_type().base)
+            origin = origin_field.sql_id(origin, Relation)
 
         target = getattr(Relation, self.target).sql_column(relation)
         if '.' not in name:
@@ -472,10 +493,12 @@ class Many2Many(Field):
         definition['relation'] = self.get_target().__name__
         definition['search_context'] = encoder.encode(self.search_context)
         definition['search_order'] = encoder.encode(self.search_order)
-        definition['sortable'] &= hasattr(model, 'order_' + self.name)
         definition['order'] = (
             getattr(self.get_target(), '_order', None)
             if self.order is None else self.order)
         if self.size is not None:
             definition['size'] = encoder.encode(self.size)
         return definition
+
+    def sortable(self, model):
+        return super().sortable(model) and hasattr(model, f'order_{self.name}')
