@@ -1,7 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
-from trytond.model import ModelSQL, fields
+from trytond.model import Check, ModelSQL, fields
 from trytond.pool import Pool
 
 
@@ -27,6 +27,21 @@ class BinaryRequired(ModelSQL):
     binary = fields.Binary('Binary Required', required=True)
 
 
+class BinaryRequiredSQLConstraint(ModelSQL):
+    "Binary Required SQL Constraint"
+    __name__ = 'test.binary_required_sql_constraint'
+    binary = fields.Binary('Binary Required', required=True)
+    constraint = fields.Boolean("Constraint")
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        t = cls.__table__()
+        cls._sql_constraints.append(
+            ('constraint', Check(t, t.constraint),
+                'tests.msg_binary_required_sql_constraint'))
+
+
 class BinaryFileStorage(ModelSQL):
     "Binary in FileStorage"
     __name__ = 'test.binary_filestorage'
@@ -39,5 +54,6 @@ def register(module):
         Binary,
         BinaryDefault,
         BinaryRequired,
+        BinaryRequiredSQLConstraint,
         BinaryFileStorage,
         module=module, type_='model')
