@@ -1,7 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
-from trytond.model import ModelSQL, DictSchemaMixin, fields
+from trytond.model import DictSchemaMixin, ModelSQL, fields
 from trytond.pool import Pool
 
 
@@ -34,10 +34,34 @@ class DictRequired(ModelSQL):
     dico = fields.Dict(None, 'Test Dict', required=True)
 
 
-class DictJSONB(ModelSQL):
-    'Dict JSONB'
-    __name__ = 'test.dict_jsonb'
+class DictText(ModelSQL):
+    'Dict TEXT'
+    __name__ = 'test.dict_text'
     dico = fields.Dict('test.dict.schema', 'Test Dict')
+    dico._sql_type = 'TEXT'
+
+
+class DictNoSchema(ModelSQL):
+    "Dict No Schema"
+    __name__ = 'test.dict_noschema'
+    dico = fields.Dict(None, "Dict")
+
+
+class DictUnaccentedOn(ModelSQL):
+    "Dict Unaccented On"
+    __name__ = 'test.dict_unaccented_on'
+    dico = fields.Dict(None, "Dict")
+
+
+class DictUnaccentedOff(ModelSQL):
+    "Dict Unaccented Off"
+    __name__ = 'test.dict_unaccented_off'
+    dico = fields.Dict(None, "Dict")
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.dico.search_unaccented = False
 
 
 def register(module):
@@ -46,5 +70,8 @@ def register(module):
         Dict,
         DictDefault,
         DictRequired,
-        DictJSONB,
+        DictText,
+        DictNoSchema,
+        DictUnaccentedOn,
+        DictUnaccentedOff,
         module=module, type_='model')
