@@ -2,23 +2,32 @@
 # this repository contains the full copyright notices and license terms.
 
 import datetime as dt
+import os
 import time
 import unittest
 
-from trytond import backend, cache as cache_mod
-from trytond.cache import freeze, MemoryCache, LRUDict, LRUDictTransaction
-from trytond.tests.test_tryton import with_transaction, activate_module
-from trytond.tests.test_tryton import DB_NAME, USER
+from trytond import backend
+from trytond import cache as cache_mod
+from trytond.cache import (
+    LRUDict, LRUDictTransaction, MemoryCache, freeze, unfreeze)
+from trytond.tests.test_tryton import (
+    DB_NAME, USER, activate_module, with_transaction)
 from trytond.transaction import Transaction
 
-
 cache = MemoryCache('test.cache')
 cache_expire = MemoryCache('test.cache_expire', duration=1)
+cache_ignored_local_context = MemoryCache(
+    'test.cache.ignored.local', context_ignored_keys={'ignored'})
+cache_ignored_global_context = MemoryCache('test.cache.ignored.global')
 
 
 class CacheTestCase(unittest.TestCase):
     "Test Cache"
 
+    @classmethod
+    def setUpClass(cls):
+        activate_module('tests')
+
     def testFreeze(self):
         "Test freeze"
         self.assertEqual(freeze([1, 2, 3]), (1, 2, 3))
@@ -41,6 +50,50 @@ class CacheTestCase(unittest.TestCase):
                                             ('string', 'test'),
                                             ]))]))]))
 
+    def testUnfreeze(self):
+        "Test unfreeze"
+        for value, result in [
+                (freeze([1, 2, 3]), [1, 2, 3]),
+                (freeze({'dict': {
+                                'inner dict': {
+                                    'list': [1, 2, 3],
+                                    'string': 'test',
+                                    },
+                                },
+                            }),
+                    {'dict': {
+                            'inner dict': {
+                                'list': [1, 2, 3],
+                                'string': 'test',
+                                },
+                            },
+                        }),
+                ]:
+            with self.subTest(value=value):
+                self.assertEqual(unfreeze(value), result)
+
+    @with_transaction()
+    def test_ignored_context_key_global(self):
+        "Test global keys are ignored from context"
+        with Transaction().set_context(client='foo'):
+            cache_ignored_global_context.set('key', 0)
+
+        with Transaction().set_context(client='bar'):
+            value = cache_ignored_global_context.get('key')
+
+        self.assertEqual(value, 0)
+
+    @with_transaction()
+    def test_ignored_context_key_local(self):
+        "Test local keys are ignored from context"
+        with Transaction().set_context(ignored='foo'):
+            cache_ignored_local_context.set('key', 1)
+
+        with Transaction().set_context(ignored='bar'):
+            value = cache_ignored_local_context.get('key')
+
+        self.assertEqual(value, 1)
+
 
 class MemoryCacheTestCase(unittest.TestCase):
     "Test Cache"
@@ -59,6 +112,9 @@ class MemoryCacheTestCase(unittest.TestCase):
     def tearDown(self):
         MemoryCache.drop(DB_NAME)
 
+    def wait_cache_listening(self):
+        pass
+
     def wait_cache_sync(self, after=None):
         pass
 
@@ -89,6 +145,7 @@ class MemoryCacheTestCase(unittest.TestCase):
     def test_memory_cache_transactions(self):
         "Test MemoryCache with concurrent transactions"
         transaction1 = Transaction().start(DB_NAME, USER)
+        self.wait_cache_listening()
         self.addCleanup(transaction1.stop)
 
         cache.set('foo', 'bar')
@@ -137,6 +194,7 @@ class MemoryCacheTestCase(unittest.TestCase):
     def test_memory_cache_old_transaction(self):
         "Test old transaction does not fill cache"
         transaction1 = Transaction().start(DB_NAME, USER)
+        self.wait_cache_listening()
         self.addCleanup(transaction1.stop)
 
         # Clear cache from new transaction
@@ -183,6 +241,14 @@ class MemoryCacheChannelTestCase(MemoryCacheTestCase):
         while MemoryCache._clean_last < after:
             time.sleep(.01)
 
+    def wait_cache_listening(self):
+        pid = os.getpid()
+        dbname = Transaction().database.name
+        listener = MemoryCache._listener.get((pid, dbname))
+        while (not getattr(listener, 'listening', False)
+                and listener.is_alive()):
+            time.sleep(.01)
+
     @unittest.skip("No cache sync on transaction start with channel")
     def test_memory_cache_sync(self):
         super().test_memory_cache_sync()
@@ -230,6 +296,12 @@ class LRUDictTestCase(unittest.TestCase):
         lru_dict['bar'].append('bar')
         self.assertEqual(lru_dict, {'bar': ['bar']})
 
+    def test_default_factory_with_key(self):
+        lru_dict = LRUDict(
+            1, default_factory=lambda k: k, default_factory_with_key=True)
+
+        self.assertEqual(lru_dict['foo'], 'foo')
+
 
 class LRUDictTransactionTestCase(unittest.TestCase):
     "Test LRUDictTransaction"
@@ -270,17 +342,3 @@ class LRUDictTransactionTestCase(unittest.TestCase):
 
         self.assertEqual(lru_dict, {})
         self.assertEqual(lru_dict.counter, Transaction().counter)
-
-
-def suite():
-    func = unittest.TestLoader().loadTestsFromTestCase
-    suite = unittest.TestSuite()
-    for testcase in [
-            CacheTestCase,
-            MemoryCacheTestCase,
-            MemoryCacheChannelTestCase,
-            LRUDictTestCase,
-            LRUDictTransactionTestCase,
-            ]:
-        suite.addTests(func(testcase))
-    return suite
