@@ -6,7 +6,7 @@ from decimal import Decimal, InvalidOperation
 from sql import Literal
 
 from trytond.model.exceptions import (
-    RequiredValidationError, DigitsValidationError)
+    DigitsValidationError, RequiredValidationError)
 from trytond.pool import Pool
 from trytond.tests.test_tryton import activate_module, with_transaction
 
@@ -70,6 +70,16 @@ class FieldNumericTestCase(unittest.TestCase):
 
         self.assertEqual(numeric.numeric, Decimal('2.5'))
 
+    @with_transaction()
+    def test_set_sql_value(self):
+        "Test cannot set SQL value"
+        Numeric = Pool().get('test.numeric')
+
+        numeric = Numeric()
+
+        with self.assertRaises(ValueError):
+            numeric.numeric = Literal(42)
+
     @with_transaction()
     def test_create_non_numeric(self):
         "Test create numeric with non numeric"
@@ -559,7 +569,3 @@ class FieldNumericTestCase(unittest.TestCase):
             Numeric.write([numeric], {
                     'digits': 0,
                     })
-
-
-def suite():
-    return unittest.TestLoader().loadTestsFromTestCase(FieldNumericTestCase)
