@@ -6,10 +6,11 @@ from unittest.mock import patch
 
 from lxml import etree
 
-from trytond.model.exceptions import AccessError, AccessButtonError
-from trytond.tests.test_tryton import activate_module, with_transaction
+from trytond.model.exceptions import (
+    AccessButtonError, AccessError, ButtonActionException)
 from trytond.pool import Pool
-from trytond.pyson import PYSONEncoder, PYSONDecoder, Eval
+from trytond.pyson import Eval, PYSONDecoder, PYSONEncoder
+from trytond.tests.test_tryton import activate_module, with_transaction
 
 
 class ModelView(unittest.TestCase):
@@ -108,6 +109,22 @@ class ModelView(unittest.TestCase):
                     },
                 })
 
+    @with_transaction()
+    def test_changed_values_2many_function(self):
+        "Test changed values for xxx2many function field"
+        pool = Pool()
+        Model = pool.get('test.modelview.changed_values')
+
+        with patch.object(
+                Model, 'on_change_with_m2m_function') as on_change_with:
+            on_change_with.return_value = [42]
+            record = Model()
+            record.m2m_function = record.on_change_with_m2m_function()
+
+            self.assertEqual(record._changed_values, {
+                    'm2m_function': {'add': [(0, {'id': 42})]},
+                    })
+
     @with_transaction()
     def test_changed_values_stored(self):
         "Test stored changed values"
@@ -668,10 +685,81 @@ class ModelView(unittest.TestCase):
 
         self.assertIn('bar', fields)
 
+    @with_transaction()
+    def test_states_depends_display(self):
+        "Test that fields used in readonly states are included"
+        pool = Pool()
+        Depends = pool.get('test.modelview.states_depends')
 
-def suite():
-    func = unittest.TestLoader().loadTestsFromTestCase
-    suite = unittest.TestSuite()
-    for testcase in (ModelView,):
-        suite.addTests(func(testcase))
-    return suite
+        readonly_fields = Depends.fields_view_get(view_type='tree')['fields']
+        self.assertIn('bar', readonly_fields)
+        self.assertNotIn('baz', readonly_fields)
+        self.assertIn('quux', readonly_fields)
+
+    @with_transaction()
+    def test_states_depends_edition(self):
+        "Test that fields used in editable states are included"
+        pool = Pool()
+        Depends = pool.get('test.modelview.states_depends')
+
+        writeable_fields = Depends.fields_view_get(view_type='form')['fields']
+        self.assertIn('bar', writeable_fields)
+        self.assertIn('baz', writeable_fields)
+        self.assertIn('quux', writeable_fields)
+
+    @with_transaction()
+    def test_button_action_exception(self):
+        "Test value of ButtonActionException"
+        exception = ButtonActionException('tests.test_modelview_button_action')
+        self.assertEqual(exception.value, {
+                **exception.value,
+                'name': "Test Button Action",
+                'url': 'http://www.example.com/',
+                })
+
+    @with_transaction()
+    def test_button_action_exception_value(self):
+        "Test value of ButtonActionException"
+        exception = ButtonActionException(
+            'tests.test_modelview_button_action',
+            value={
+                'url': 'https://www.example.net',
+                })
+        self.assertEqual(exception.value, {
+                **exception.value,
+                'name': "Test Button Action",
+                'url': 'https://www.example.net',
+                })
+
+    @with_transaction()
+    def test_autocomplete(self):
+        "Test autocomplete"
+        pool = Pool()
+        Model = pool.get('test.modelview.autocomplete')
+
+        self.assertEqual(Model.autocomplete('test'), [])
+
+    @with_transaction()
+    def test_autocomplete_storage(self):
+        "Test autocomplete storage"
+        pool = Pool()
+        Model = pool.get('test.modelview.autocomplete.storage')
+
+        foo, bar = Model.create([{'name': 'foo'}, {'name': 'bar'}])
+
+        self.assertEqual(Model.autocomplete('test'), [])
+        self.assertEqual(
+            Model.autocomplete('foo'),
+            [{'name': 'foo', 'id': foo.id, 'defaults': None}])
+        self.assertEqual(
+            Model.autocomplete(''), [
+                {'name': 'foo', 'id': foo.id, 'defaults': None},
+                {'name': 'bar', 'id': bar.id, 'defaults': None}])
+        self.assertEqual(Model.autocomplete('foo', [('id', '!=', foo.id)]), [])
+        self.assertEqual(
+            Model.autocomplete('', limit=1),
+            [{'name': 'foo', 'id': foo.id, 'defaults': None}])
+        self.assertEqual(
+            Model.autocomplete('', order=[('name', 'ASC')]), [
+                {'name': 'bar', 'id': bar.id, 'defaults': None},
+                {'name': 'foo', 'id': foo.id, 'defaults': None}])
