@@ -4,10 +4,10 @@
 import unittest
 from unittest.mock import Mock, sentinel
 
-from werkzeug.wrappers import Response
-from werkzeug.test import Client
-from trytond.wsgi import TrytondWSGI
 from trytond.exceptions import TrytonException
+from trytond.protocols.wrappers import Response
+from trytond.tests.test_tryton import Client
+from trytond.wsgi import TrytondWSGI
 
 
 class WSGIAppTestCase(unittest.TestCase):
@@ -109,10 +109,3 @@ class WSGIAppTestCase(unittest.TestCase):
 
         self.assertEqual(next(response.response), b'baz')
         self.assertEqual(response.status, "418 I'M A TEAPOT")
-
-
-def suite():
-    suite_ = unittest.TestSuite()
-    suite_.addTests(
-        unittest.TestLoader().loadTestsFromTestCase(WSGIAppTestCase))
-    return suite_
