@@ -1,7 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
-from trytond.model import ModelSQL, tree, fields
+from trytond.model import ModelSQL, fields, tree
 from trytond.pool import Pool
 
 
