@@ -1,8 +1,7 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from .wizard import Wizard
-from .wizard import StateView, StateTransition, StateAction, StateReport
-from .wizard import Button
+from .wizard import (
+    Button, StateAction, StateReport, StateTransition, StateView, Wizard)
 
 __all__ = ['Wizard',
     'StateView', 'StateTransition', 'StateAction', 'StateReport',
